#include <EasyTransfer.h>

//create two objects
EasyTransfer ETin, ETout; 

struct RECEIVE_DATA_STRUCTURE
{
  byte brightness;
  char ledLevel[14];
  char motionDetected[14];
  byte boardNumber;
};

struct SEND_DATA_STRUCTURE
{
  byte brightness;
  char ledLevel[14];  
  char motionDetected[14];  
  byte boardNumber;
};

byte masterPin = 2;
byte motionPin = A4;
byte motionPin2 = A2; // entrance/exit pins
byte brightness = 0;    // how bright the LED is
byte fadeAmount = 5;    // how many points to fade the LED by

boolean boolMaster = false;

byte boardNumber;

RECEIVE_DATA_STRUCTURE rxdata;
SEND_DATA_STRUCTURE txdata;

void setup()
{
  Serial.begin(57600);
  ETin.begin(details(rxdata), &Serial);
  ETout.begin(details(txdata), &Serial);

  pinMode(masterPin, INPUT);

  pinMode(motionPin, INPUT);
  pinMode(motionPin2, INPUT);

  pinMode(13, OUTPUT); // id number led.

  pinMode(3, OUTPUT); // pwm pins for LED strip
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);
  pinMode(9, OUTPUT);

  for(int i = 7; i<=8; i++)
  {
    pinMode(i, OUTPUT);
    digitalWrite(i, LOW);
  }

  for(int i = 10; i<=12; i++)
  {
    pinMode(i, OUTPUT);
    digitalWrite(i, LOW);
  }

  pinMode(A5, OUTPUT);

  if(digitalRead(masterPin)) // if A7 = GND, then Master
  {
    boolMaster = true;
    boardNumber = 1;
  }
  else // if A7 = 5V, then Slave
  boolMaster = false;
}

void loop()
{
  if(boolMaster) // if i am the master!
  {
    txdata.boardNumber = 2; // assign number 2 to the first slave.
    ETout.sendData();

    if(ETin.receiveData())
    {
      masterCommands();
    }
  }
  else // if i am a slave... :(
  {
    if(ETin.receiveData())
    {
      boardNumber = rxdata.boardNumber; // get what board number.
      // add 1, and assign to the other board.
      txdata.boardNumber = boardNumber + 1;

      led1(rxdata.ledLevel[boardNumber]); // light strip pwm

      // re-transmit motionDetected status. stacks all of them
      // up to the last board for the master to read.
      if(boardNumber > 2)
      {
        for(int i = 2; i < boardNumber; i++)
        {
          txdata.motionDetected[i] = rxdata.motionDetected[i];
        }
      }
      // re-transmit led light values from the master to each slave.
      for(int i = boardNumber; i <= 12; i++)
        txdata.ledLevel[i] = rxdata.ledLevel[i];

      task_motionSense();

      ETout.sendData();
    }
  } // end of 'else'
  task_blinkMaster();
}

void task_motionSense()
{
  if(digitalRead(motionPin) == LOW)
  {
    txdata.motionDetected[boardNumber] = 1;
    //    ETout.sendData();
  }
  else
  {
    txdata.motionDetected[boardNumber] = 0;
    //    ETout.sendData();
  }

  if(boardNumber == 12) // if i'm the last board. // change to 12.
  {
    // check exit/entrance sensor
    if(digitalRead(motionPin2) == LOW)
    {
      txdata.motionDetected[boardNumber+1] = 1; // send board number 13.
    }
    else
    {
      txdata.motionDetected[boardNumber+1] = 0;      
    }
  }
}
/*
 performs non-blocking on-board led blink to know
 its board number.
 1 blink - board number 1 (master)
 2 blinks - board number 2 (slave)
 3 blinks - board number 3 (slave) and so on..
 */
unsigned long time_blinkMaster;
void task_blinkMaster()
{
  static byte blink_count = 0;

  if(millis() - time_blinkMaster > 250)
    time_blinkMaster = millis();
  else
    return;
  blink_count++;
  if(blink_count < boardNumber * 2)
    digitalWrite(13, !digitalRead(13)); // toggle 13 pin.
  else
    digitalWrite(13, LOW);

  if(blink_count > (boardNumber * 2) + 5)
    blink_count = 0;
}

void led1 (int analogValue)
{
  analogWrite(3, analogValue);
  analogWrite(5, analogValue);
  analogWrite(6, analogValue);
  analogWrite(9, analogValue);
}


//------------ Animation, Main program -----------------------------

// lowest fade value. if set to 0, then all lights stay off without
// sensor.
int lowestFadeValue[] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0};

// makes maximum brightness for each light adjustable.
int highestFadeValue[] = {
  255,255,255,255,255,255,255,255,255,255,255,255,255};

int ledLevel[] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0};
// fade_skipValue = amount of delay
// fade_InorOut = direction, 1 = fade in, 0 = fade out.
byte fade(byte fade_skipValue, boolean fade_InorOut, byte maxFade, byte minFade, byte stepNumber)
{
  static byte stateVar[] = {
    0,0,0,0,0,0,0,0,0,0,0,0,0                      };
  switch(stateVar[stepNumber])
  {
  case 0: // determine if fade in or fade out
    if(fade_InorOut == 1)
      stateVar[stepNumber] = 1;
    else
      stateVar[stepNumber] = 2;
    break;

  case 1: // fade in
    ledLevel[stepNumber] += fade_skipValue;

    highestFadeValue[stepNumber] = maxFade;

    if(ledLevel[stepNumber] >= highestFadeValue[stepNumber])
      stateVar[stepNumber] = 3;
    else
    {
      if(stepNumber == 1)
        led1(ledLevel[stepNumber]);
      else        
        txdata.ledLevel[stepNumber] = ledLevel[stepNumber];

      ETout.sendData();
      stateVar[stepNumber] = 0; // go back to 1st state.
    }
    break;

  case 2: // fade out
    ledLevel[stepNumber] -= fade_skipValue;

    lowestFadeValue[stepNumber] = minFade;

    if(ledLevel[stepNumber] <= lowestFadeValue[stepNumber])
      stateVar[stepNumber] = 4;
    else
    {
      if(stepNumber == 1)
        led1(ledLevel[stepNumber]);
      else
        txdata.ledLevel[stepNumber] = ledLevel[stepNumber];

      ETout.sendData();
      stateVar[stepNumber] = 0; // go back to 1st state.
    }
    break;

  case 3: // if ledLevel is max (255), then stay max.
    if(fade_InorOut == 1)
    {
      highestFadeValue[stepNumber] = maxFade;
      ledLevel[stepNumber] = highestFadeValue[stepNumber];

      if(stepNumber == 1)
        led1(ledLevel[stepNumber]);
      else
        txdata.ledLevel[stepNumber] = ledLevel[stepNumber];

      ETout.sendData(); 
    }
    else
      stateVar[stepNumber] = 0;
    break;

  case 4: // if ledLevel is min (0), then stay min.
    if(fade_InorOut == 0)
    {
      lowestFadeValue[stepNumber] = minFade;

      ledLevel[stepNumber] = lowestFadeValue[stepNumber];

      if(stepNumber == 1)
        led1(ledLevel[stepNumber]);
      else
        txdata.ledLevel[stepNumber] = ledLevel[stepNumber];

      ETout.sendData(); 
    }
    else
      stateVar[stepNumber] = 0;
    break;

    // just for safe keeping. if stateVar goes out of control
    // program goes here.
  default:
    stateVar[stepNumber] = 0;
    break; 
  }

  return ledLevel[stepNumber];
}

//------------------------------------------------------------------



void masterCommands()
{
  animate_LightChaser();
}

byte currentStep;
byte prevStep;
boolean directionStep;

void animate_LightChaser()
{
  if(digitalRead(motionPin) == 0 || digitalRead(motionPin2) == 0)
  {
    fade(15,1,255,1,1);
    fade(15,1,255,1,2);
    fade(15,1,255,1,3);
    fade(10,0,255,1,4);
    fade(10,0,255,1,5);
    fade(10,0,255,1,6);
    fade(10,0,255,1,7);
    fade(10,0,255,1,8);
    fade(10,0,255,1,9);
    fade(10,0,255,1,10);
    fade(10,0,255,1,11);
    fade(10,0,255,1,12);

    currentStep = 1;
  }
  else if(rxdata.motionDetected[2] == 1)
  {
    fade(15,1,255,1,1);
    fade(15,1,255,1,2);
    fade(15,1,255,1,3);
    fade(10,0,255,1,4);
    fade(10,0,255,1,5);
    fade(10,0,255,1,6);
    fade(10,0,255,1,7);
    fade(10,0,255,1,8);
    fade(10,0,255,1,9);
    fade(10,0,255,1,10);
    fade(10,0,255,1,11);
    fade(10,0,255,1,12);

    currentStep = 2;    
  }
  else if(rxdata.motionDetected[3] == 1)
  {
    fade(10,0,255,1,1);
    fade(15,1,255,1,2);
    fade(15,1,255,1,3);
    fade(15,1,255,1,4);
    fade(10,0,255,1,5);
    fade(10,0,255,1,6);
    fade(10,0,255,1,7);
    fade(10,0,255,1,8);
    fade(10,0,255,1,9);
    fade(10,0,255,1,10);
    fade(10,0,255,1,11);
    fade(10,0,255,1,12);    

    currentStep = 3;    
  }
  else if(rxdata.motionDetected[4] == 1)
  {
    fade(10,0,255,1,1);
    fade(10,0,255,1,2);
    fade(15,1,255,1,3);
    fade(15,1,255,1,4);
    fade(15,1,255,1,5);
    fade(10,0,255,1,6);
    fade(10,0,255,1,7);
    fade(10,0,255,1,8);
    fade(10,0,255,1,9);
    fade(10,0,255,1,10);
    fade(10,0,255,1,11);
    fade(10,0,255,1,12);    

    currentStep = 4;    
  }
  else if(rxdata.motionDetected[5] == 1)
  {
    fade(10,0,255,1,1);
    fade(10,0,255,1,2);
    fade(10,0,255,1,3);
    fade(15,1,255,1,4);
    fade(15,1,255,1,5);
    fade(15,1,255,1,6);
    fade(10,0,255,1,7);
    fade(10,0,255,1,8);
    fade(10,0,255,1,9);
    fade(10,0,255,1,10);
    fade(10,0,255,1,11);
    fade(10,0,255,1,12);    

    currentStep = 5;    
  }
  else if(rxdata.motionDetected[6] == 1)
  {
    fade(10,0,255,1,1);
    fade(10,0,255,1,2);
    fade(10,0,255,1,3);
    fade(10,0,255,1,4);
    fade(15,1,255,1,5);
    fade(15,1,255,1,6);
    fade(15,1,255,1,7);
    fade(10,0,255,1,8);
    fade(10,0,255,1,9);
    fade(10,0,255,1,10);
    fade(10,0,255,1,11);
    fade(10,0,255,1,12);    

    currentStep = 6;
  }
  else if(rxdata.motionDetected[7] == 1)
  {
    fade(10,0,255,1,1);
    fade(10,0,255,1,2);
    fade(10,0,255,1,3);
    fade(10,0,255,1,4);
    fade(10,0,255,1,5);
    fade(15,1,255,1,6);
    fade(15,1,255,1,7);
    fade(15,1,255,1,8);
    fade(10,0,255,1,9);
    fade(10,0,255,1,10);
    fade(10,0,255,1,11);
    fade(10,0,255,1,12);    

    currentStep = 7;
  }
  else if(rxdata.motionDetected[8] == 1)
  {
    fade(10,0,255,1,1);
    fade(10,0,255,1,2);
    fade(10,0,255,1,3);
    fade(10,0,255,1,4);
    fade(10,0,255,1,5);
    fade(10,0,255,1,6);
    fade(15,1,255,1,7);
    fade(15,1,255,1,8);
    fade(15,1,255,1,9);
    fade(10,0,255,1,10);
    fade(10,0,255,1,11);
    fade(10,0,255,1,12);    

    currentStep = 8;
  } 
  else if(rxdata.motionDetected[9] == 1)
  {
    fade(10,0,255,1,1);
    fade(10,0,255,1,2);
    fade(10,0,255,1,3);
    fade(10,0,255,1,4);
    fade(10,0,255,1,5);
    fade(10,0,255,1,6);
    fade(10,0,255,1,7);
    fade(15,1,255,1,8);
    fade(15,1,255,1,9);
    fade(15,1,255,1,10);
    fade(10,0,255,1,11);
    fade(10,0,255,1,12);     

    currentStep = 9;
  }
  else if(rxdata.motionDetected[10] == 1)
  {
    fade(10,0,255,1,1);
    fade(10,0,255,1,2);
    fade(10,0,255,1,3);
    fade(10,0,255,1,4);
    fade(10,0,255,1,5);
    fade(10,0,255,1,6);
    fade(10,0,255,1,7);
    fade(10,0,255,1,8);
    fade(15,1,255,1,9);
    fade(15,1,255,1,10);
    fade(15,1,255,1,11);
    fade(10,0,255,1,12);      

    currentStep = 10;
  }
  else if(rxdata.motionDetected[11] == 1)
  {
    fade(10,0,255,1,1);
    fade(10,0,255,1,2);
    fade(10,0,255,1,3);
    fade(10,0,255,1,4);
    fade(10,0,255,1,5);
    fade(10,0,255,1,6);
    fade(10,0,255,1,7);
    fade(10,0,255,1,8);
    fade(10,0,255,1,9);
    fade(15,1,255,1,10);
    fade(15,1,255,1,11);
    fade(15,1,255,1,12);    

    currentStep = 11;
  }
  else if(rxdata.motionDetected[12] == 1 || rxdata.motionDetected[13] == 1)
  {
    fade(10,0,255,1,1);
    fade(10,0,255,1,2);
    fade(10,0,255,1,3);
    fade(10,0,255,1,4);
    fade(10,0,255,1,5);
    fade(10,0,255,1,6);
    fade(10,0,255,1,7);
    fade(10,0,255,1,8);
    fade(10,0,255,1,9);
    fade(10,1,255,1,10);
    fade(15,1,255,1,11);
    fade(15,1,255,1,12);      

    currentStep = 12;
  }
  else
  {
    if(directionStep == 1)
    {
      //fade out going up
      if(fade(3,0,255,1,1) == 1)
      {
        if(fade(10,0,255,1,2) == 1)
        {
          if(fade(10,0,255,1,3) == 1)
          {
            if(fade(8,0,255,1,4) == 1)
            {
              if(fade(8,0,255,1,5) == 1)
              {
                if(fade(8,0,255,1,6) == 1)
                {
                  if(fade(8,0,255,1,7) == 1)
                  {
                    if(fade(8,0,255,1,8) == 1)
                    {
                      if(fade(9,0,255,1,9) == 1)
                      {
                        if(fade(12,0,255,1,10) == 1) 
                        {
                          if(fade(14,0,255,1,11) == 1)
                          {
                            if(fade(15,0,255,1,12) == 1);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    else if(directionStep == 0)
    {
      if(fade(8,0,255,1,12) == 1)
      {
        if(fade(8,0,255,1,11) == 1)
        {
          if(fade(10,0,255,1,10) == 1)
          {
            if(fade(8,0,255,1,9) == 1)
            {
              if(fade(8,0,255,1,8) == 1)
              {
                if(fade(8,0,255,1,7) == 1)
                {
                  if(fade(8,0,255,1,6) == 1)
                  {
                    if(fade(8,0,255,1,5) == 1)
                    {
                      if(fade(8,0,255,1,4) == 1)
                      {
                        if(fade(8,0,255,1,3) == 1) 
                        {
                          if(fade(8,0,255,1,2) == 1)
                          {
                            if(fade(8,0,255,1,1) == 1);
                          }
                        }
                      }
                    }
                  }
                } // 6
              }
            }
          }
        }
      }

    }
  } // end of 'else'

  if(prevStep - currentStep < 0)
    directionStep = 1;
  else if(prevStep - currentStep > 0)
    directionStep = 0;


  prevStep = currentStep;
}







